@@ -1,4 +1,4 @@
-#  -*- coding: utf-8 -*-
+# -*- coding: utf-8 -*-
 # SPDX-License-Identifier: MPL-2.0
 # Copyright 2020-2021 John Mille <john@compose-x.io>
 
@@ -27,7 +27,9 @@ from ecs_composex.dashboards.dashboards_stack import XStack as DashboardsStack
 from ecs_composex.dns import DnsSettings
 from ecs_composex.dns.dns_records import DnsRecords
 from ecs_composex.ecs.ecs_cluster import add_ecs_cluster
+from ecs_composex.ecs.ecs_params import CLUSTER_NAME
 from ecs_composex.ecs.ecs_stack import associate_services_to_root_stack
+from ecs_composex.iam.iam_stack import XStack as IamStack
 from ecs_composex.vpc import vpc_params
 from ecs_composex.vpc.vpc_stack import add_vpc_to_root
 
@@ -139,13 +141,14 @@ def get_mod_class(module_name):
     return the_class
 
 
-def invoke_x_to_ecs(module_name, settings, services_stack, resource):
+def invoke_x_to_ecs(module_name, services_stack, resource, settings):
     """
     Function to associate X resources to Services
 
+    :param None,str module_name: The name of the module managing the resource type
     :param ecs_composex.common.settings.ComposeXSettings settings: The compose file content
     :param ecs_composex.ecs.ServicesStack services_stack: root stack for services.
-    :param resource: The XStack resource
+    :param ecs_composex.common.stacks.ComposeXStack resource: The XStack resource of the module
     :return:
     """
     if module_name is None:
@@ -167,20 +170,25 @@ def invoke_x_to_ecs(module_name, settings, services_stack, resource):
 def apply_x_configs_to_ecs(settings, root_stack):
     """
     Function that evaluates only the x- resources of the root template and iterates over the resources.
-    If there is an implemented module in ECS ComposeX for that resource to map to the ECS Services, it will
+    If there is an implemented module in ECS ComposeX for that resource_stack to map to the ECS Services, it will
     execute the function available in the module to apply defined settings to the services stack.
 
+    The root_stack is used as the parent stack to the services.
+
     :param ecs_composex.common.settings.ComposeXSettings settings: The compose file content
     :param ecs_composex.ecs.ServicesStack root_stack: root stack for services.
     """
     for resource_name in root_stack.stack_template.resources:
-        resource = root_stack.stack_template.resources[resource_name]
+        resource_stack = root_stack.stack_template.resources[resource_name]
         if (
-            issubclass(type(resource), ComposeXStack)
-            and resource.name in SUPPORTED_X_MODULE_NAMES
-            and not resource.is_void
+            issubclass(type(resource_stack), ComposeXStack)
+            and resource_stack.name in SUPPORTED_X_MODULE_NAMES
+            and not resource_stack.is_void
         ):
-            invoke_x_to_ecs(None, settings, root_stack, resource)
+            invoke_x_to_ecs(None, root_stack, resource_stack, settings)
+    for resource_stack in settings.x_resources_void:
+        res_type = list(resource_stack.keys())[-1]
+        invoke_x_to_ecs(res_type, root_stack, resource_stack[res_type], settings)
 
 
 def apply_x_to_x_configs(root_stack, settings):
@@ -250,7 +258,7 @@ def handle_new_xstack(
 
     LOG.debug(xstack, xstack.is_void)
     if xstack.is_void:
-        invoke_x_to_ecs(res_type, settings, services_stack, xstack)
+        settings.x_resources_void.append({res_type: xstack})
     elif (
         hasattr(xstack, "title")
         and hasattr(xstack, "stack_template")
@@ -391,6 +399,23 @@ def evaluate_ecr_configs(settings):
     return result
 
 
+def set_container_cluster_identifier(root_stack, settings):
+    """
+    Final pass at the top stacks parameters to set the ECS cluster parameter
+
+    :param ecs_composex.common.stacks.ComposeXStack root_stack:
+    :param ecs_composex.common.settings.ComposeXSettings settings:
+    """
+    for name, resource in root_stack.stack_template.resources.items():
+        if issubclass(type(resource), ComposeXStack):
+            if CLUSTER_NAME.title in [
+                param.title for param in resource.stack_template.parameters.values()
+            ]:
+                resource.Parameters.update(
+                    {CLUSTER_NAME.title: settings.ecs_cluster.cluster_identifier}
+                )
+
+
 def generate_full_template(settings):
     """
     Function to generate the root root_template
@@ -410,21 +435,23 @@ def generate_full_template(settings):
     vpc_stack = add_vpc_to_root(root_stack, settings)
     settings.set_networks(vpc_stack, root_stack)
     dns_settings = DnsSettings(root_stack, settings, get_vpc_id(vpc_stack))
-    add_ecs_cluster(root_stack, settings)
-    associate_services_to_root_stack(root_stack, settings, vpc_stack)
-    if keyisset(ACM_KEY, settings.compose_content):
-        init_acm_certs(settings, dns_settings, root_stack)
     add_x_resources(
         root_stack.stack_template,
         settings,
         root_stack,
         vpc_stack=vpc_stack,
     )
+    add_ecs_cluster(root_stack, settings)
+    iam_stack = IamStack("iam", settings)
+    root_stack.stack_template.add_resource(iam_stack)
+    associate_services_to_root_stack(root_stack, settings, vpc_stack)
+    set_services_alarms(settings)
+    if keyisset(ACM_KEY, settings.compose_content):
+        init_acm_certs(settings, dns_settings, root_stack)
     apply_x_configs_to_ecs(
         settings,
         root_stack,
     )
-    apply_x_to_x_configs(root_stack, settings)
     if settings.use_appmesh:
         mesh = Mesh(
             settings.compose_content["x-appmesh"],
@@ -436,10 +463,15 @@ def generate_full_template(settings):
     dns_records = DnsRecords(settings)
     dns_records.associate_records_to_resources(settings, root_stack, dns_settings)
     dns_settings.associate_settings_to_nested_stacks(root_stack)
-    set_services_alarms(settings)
     if keyisset("x-dashboards", settings.compose_content):
         DashboardsStack("dashboards", settings)
     add_all_tags(root_stack.stack_template, settings)
     for family in settings.families.values():
         family.validate_compute_configuration_for_task(settings)
+        family.set_enable_execute_command()
+        if family.enable_execute_command:
+            family.apply_ecs_execute_command_permissions(settings)
+        family.wait_for_all_policies()
+    apply_x_to_x_configs(root_stack, settings)
+    set_container_cluster_identifier(root_stack, settings)
     return root_stack
