@@ -1,23 +1,19 @@
-﻿#  -*- coding: utf-8 -*-
+#  -*- coding: utf-8 -*-
 # SPDX-License-Identifier: MPL-2.0
 # Copyright 2020-2021 John Mille <john@compose-x.io>
 
 import re
 
-from troposphere import AWS_NO_VALUE, GetAtt, Ref
+from compose_x_common.compose_x_common import keyisset
+from troposphere import AWS_NO_VALUE, Ref
 from troposphere.ecs import LoadBalancer as EcsLb
-from troposphere.elasticloadbalancingv2 import (
-    Matcher,
-    TargetGroup,
-    TargetGroupAttribute,
-)
+from troposphere.elasticloadbalancingv2 import Matcher, TargetGroupAttribute
 
-from ecs_composex.common import LOG, add_parameters, keyisset
-from ecs_composex.common.cfn_params import Parameter
-from ecs_composex.common.outputs import ComposeXOutput
+from ecs_composex.common import LOG, add_parameters
 from ecs_composex.ecs.ecs_params import ELB_GRACE_PERIOD
-from ecs_composex.elbv2.elbv2_params import TGT_GROUP_ARN
-from ecs_composex.vpc.vpc_params import SG_ID_TYPE, VPC_ID
+from ecs_composex.elbv2.elbv2_params import LB_SG_ID, TGT_GROUP_ARN
+from ecs_composex.elbv2.elbv2_stack import ComposeTargetGroup
+from ecs_composex.vpc.vpc_params import VPC_ID
 
 
 def validate_tcp_health_counts(props):
@@ -222,19 +218,104 @@ def handle_sg_lb_ingress_to_service(resource, family, resources_stack):
     """
     if resource.is_nlb():
         return
-    lb_sg_param = Parameter(f"{resource.lb_sg.title}", Type=SG_ID_TYPE)
+    lb_sg_param = resource.attributes_outputs[LB_SG_ID]["ImportParameter"]
     add_parameters(family.template, [lb_sg_param])
     family.service_config.network.add_lb_ingress(
         family, lb_name=resource.logical_name, lb_sg_ref=Ref(lb_sg_param)
     )
     family.stack_parameters.update(
-        {
-            f"{resource.lb_sg.title}": GetAtt(
-                resources_stack.title,
-                f"Outputs.{resource.lb_sg.title}",
-            )
-        }
+        {lb_sg_param.title: resource.attributes_outputs[LB_SG_ID]["ImportValue"]}
     )
+    if resources_stack.title not in family.stack.DependsOn:
+        family.stack.DependsOn.append(resources_stack.title)
+
+
+def validate_target_group_attributes(target_attributes, validation, lb_type):
+    """
+    Function to ensure that each attribute set is compatible with elbv2.type == application
+
+    :param list[TargetGroupAttribute] target_attributes:
+    :param dict validation:
+    :param str lb_type:
+    :raises: ValueError
+    """
+    for attr in target_attributes:
+        if attr.Key not in validation.keys():
+            raise ValueError(
+                f"Attribute {attr.Key} is not compatible with {lb_type}. Valid ones",
+                validation.keys(),
+            )
+        evaluation = validation[attr.Key]
+        if not evaluation(attr.Value):
+            raise ValueError(f"{attr.Key} value {attr.Value} is not valid.")
+
+
+def import_target_group_attributes(props, target_def, elbv2, service):
+    """
+
+    :param dict props:
+    :param dict target_def:
+    :param ecs_composex.elbv2.elbv2_stack.Elbv2 elbv2:
+    :param ecs_composex.common.compose_services.ComposeService service:
+    :return:
+    """
+    attributes_key = "TargetGroupAttributes"
+    if not keyisset(attributes_key, target_def):
+        props[attributes_key] = [
+            TargetGroupAttribute(Key="deregistration_delay.timeout_seconds", Value="60")
+        ]
+    else:
+        if isinstance(target_def[attributes_key], list):
+            props[attributes_key] = [
+                TargetGroupAttribute(Key=attr["Key"], Value=str(attr["Value"]))
+                for attr in target_def[attributes_key]
+            ]
+        elif isinstance(target_def[attributes_key], dict):
+            props[attributes_key] = [
+                TargetGroupAttribute(Key=key, Value=str(value))
+                for key, value in target_def[attributes_key].items()
+            ]
+    if not keyisset(attributes_key, props):
+        props[attributes_key] = [
+            TargetGroupAttribute(Key="deregistration_delay.timeout_seconds", Value="60")
+        ]
+        return
+    if "deregistration_delay.timeout_seconds" not in [
+        attr.Key for attr in props[attributes_key]
+    ]:
+        props[attributes_key].append(
+            TargetGroupAttribute(Key="deregistration_delay.timeout_seconds", Value="60")
+        )
+    nlb_valid = {
+        "deregistration_delay.connection_termination.enabled": lambda x: x
+        in ("true", "false"),
+        "preserve_client_ip.enabled": lambda x: x in ("true", "false"),
+        "proxy_protocol_v2.enabled": lambda x: x in ("true", "false"),
+        "stickiness.type": lambda x: x == "source_ip",
+        "deregistration_delay.timeout_seconds": lambda x: 0 <= int(x) <= 3600,
+        "stickiness.enabled": lambda x: x in ("true", "false"),
+    }
+    alb_valid = {
+        "stickiness.enabled": lambda x: x in ("true", "false"),
+        "stickiness.type": lambda x: x in ("lb_cookie", "app_cookie"),
+        "stickiness.app_cookie.cookie_name": lambda x: isinstance(x, str)
+        and not re.match(r"^AWSALB.*$|^AWSALBAPP.*|^AWSALBTG.*$", x),
+        "stickiness.app_cookie.duration_seconds": lambda x: 1 <= int(x) <= 604800,
+        "stickiness.lb_cookie.duration_seconds": lambda x: 1 <= int(x) <= 604800,
+        "deregistration_delay.timeout_seconds": lambda x: 0 <= int(x) <= 3600,
+        "load_balancing.algorithm.type": lambda x: x
+        in ("round_robin", "least_outstanding_requests"),
+        "slow_start.duration_seconds": lambda x: 30 <= int(x) <= 900,
+    }
+    # pragma: ignore use-case for now "lambda.multi_value_headers.enabled": lambda x: x in ("true", "false"),
+    if elbv2.cfn_resource.Type == "application":
+        validate_target_group_attributes(
+            props[attributes_key], alb_valid, elbv2.cfn_resource.Type
+        )
+    if elbv2.cfn_resource.Type == "network":
+        validate_target_group_attributes(
+            props[attributes_key], nlb_valid, elbv2.cfn_resource.Type
+        )
 
 
 def define_service_target_group(
@@ -246,11 +327,11 @@ def define_service_target_group(
 ):
     """
     Function to create the elbv2 target group
-    :param ecs_composex.elbv2.elbv2_stack.Elbv2 resource:
-    :param ecs_composex.common.compose_services.ComposeService service:
-    :param ecs_composex.common.compose_services.ComposeFamily family:
+    :param ecs_composex.elbv2.elbv2_stack.Elbv2 resource: the ELBv2 to attach to
+    :param ecs_composex.common.compose_services.ComposeService service: the service target
+    :param ecs_composex.common.compose_services.ComposeFamily family: the family owning the service
     :param ecs_composex.common.stacks.ComposeXStack resources_root_stack:
-    :param dict target_definition:
+    :param dict target_definition: the Service definition
     :return: the target group
     :rtype: troposphere.elasticloadbalancingv2.TargetGroup
     """
@@ -264,32 +345,28 @@ def define_service_target_group(
         else target_definition["protocol"]
     )
     props["TargetType"] = "ip"
-    props["TargetGroupAttributes"] = [
-        TargetGroupAttribute(Key="deregistration_delay.timeout_seconds", Value="60")
-    ]
+    import_target_group_attributes(props, target_definition, resource, service)
     validate_props_and_service_definition(props, service)
-    target_group = TargetGroup(
+    target_group = ComposeTargetGroup(
         f"Tgt{resource.logical_name}{family.logical_name}{service.logical_name}{props['Port']}",
+        elbv2=resource,
+        family=family,
+        stack=resource.stack,
         VpcId=Ref(VPC_ID),
         **props,
     )
+    target_group.init_outputs()
+    target_group.generate_outputs()
     resources_root_stack.stack_template.add_resource(target_group)
-    resources_root_stack.stack_template.add_output(
-        ComposeXOutput(
-            target_group,
-            [("", TGT_GROUP_ARN.title, Ref(target_group))],
-            export=False,
-        ).outputs
-    )
-    tgt_parameter = Parameter(
-        f"{target_group.title}Arn", Type="String", template=family.template
-    )
+    resources_root_stack.stack_template.add_output(target_group.outputs)
+    family.target_groups.append(target_group)
+    tgt_parameter = target_group.attributes_outputs[TGT_GROUP_ARN]["ImportParameter"]
+    add_parameters(family.template, [tgt_parameter])
     family.stack_parameters.update(
         {
-            tgt_parameter.title: GetAtt(
-                resources_root_stack.title,
-                f"Outputs.{target_group.title}{TGT_GROUP_ARN.title}",
-            )
+            tgt_parameter.title: target_group.attributes_outputs[TGT_GROUP_ARN][
+                "ImportValue"
+            ],
         }
     )
     service_lb = EcsLb(
