@@ -7,20 +7,58 @@ Module to handle resource settings definition to containers.
 """
 
 from compose_x_common.compose_x_common import keyisset
-from troposphere import FindInMap, Ref, Sub
+from troposphere import AWSHelperFn, FindInMap, Ref, Sub
 from troposphere.iam import Policy as IamPolicy
+from troposphere.iam import PolicyType
 
 from ecs_composex.common import LOG, add_parameters
-from ecs_composex.common.cfn_params import ROOT_STACK_NAME_T, STACK_ID_SHORT
+from ecs_composex.common.cfn_params import STACK_ID_SHORT
 from ecs_composex.common.compose_resources import get_parameter_settings
 from ecs_composex.common.services_helpers import extend_container_envvars
 from ecs_composex.common.stacks import ComposeXStack
 from ecs_composex.ecs.ecs_iam import define_service_containers
-from ecs_composex.ecs.ecs_params import TASK_ROLE_T
 from ecs_composex.kms.kms_perms import ACCESS_TYPES as KMS_ACCESS_TYPES
 
 
-def generate_resource_permissions(resource_name, policies, arn):
+def determine_arns(arn, policy_doc, ignore_missing_primary=False):
+    """
+    Function allowing to detect whether the resource permissions has a defined override for
+    resources ARN. This allows to extend the ARN syntax.
+
+    The policy skeleton must have Resource as a list, and contain ${ARN} into it.
+
+    :param str, list, AWSHelperFn arn:
+    :param dict policy_doc: The policy document content
+    :param bool ignore_missing_primary: Whether the policy should contain ${ARN} at least
+    :return: The list or Resource to put in to the IAM policy
+    """
+    resources = []
+    base_arn = r"${ARN}"
+    if keyisset("Resource", policy_doc):
+        if base_arn not in policy_doc["Resource"] and not ignore_missing_primary:
+            raise KeyError(
+                f"The policy skeletion must contain at least {base_arn} when Resource is defined"
+            )
+        if issubclass(type(arn), AWSHelperFn):
+            for resource in policy_doc["Resource"]:
+                if not resource.startswith(base_arn):
+                    raise ValueError(
+                        f"The value {resource} is invalid. It must start with {base_arn}"
+                    )
+                if resource == base_arn:
+                    resources.append(arn)
+                else:
+                    resources.append(Sub(f"{resource}", ARN=arn))
+        return resources
+    elif not isinstance(arn, list):
+        return [arn]
+    else:
+        return arn
+
+
+def generate_resource_permissions(
+    resource_name, policies, arn, ignore_missing_primary=False
+):
     """
     Function to generate IAM permissions for a given x-resource. Returns the mapping of these for the given resource.
     Suffix takes the values and reduces to the first 118 characters to ensure policy length is below 128
@@ -30,6 +68,7 @@ def generate_resource_permissions(resource_name, policies, arn):
     :param str resource_name: The name of the resource
     :param dict policies: the policies associated with the x-resource type.
     :param str,AWSHelper arn: The ARN of the resource if already looked up.
+    :param bool ignore_missing_primary: Whether the policy should contain ${ARN} at least
     :return: dict of the IAM policies associated with the resource.
     :rtype dict:
     """
@@ -38,8 +77,9 @@ def generate_resource_permissions(resource_name, policies, arn):
         clean_policy = {"Version": "2012-10-17", "Statement": []}
         LOG.debug(a_type)
         policy_doc = policies[a_type].copy()
-        policy_doc["Sid"] = Sub(f"{a_type}To{resource_name}")
-        policy_doc["Resource"] = [arn] if not isinstance(arn, list) else arn
+        resources = determine_arns(arn, policy_doc, ignore_missing_primary)
+        policy_doc["Sid"] = f"{a_type}To{resource_name}"
+        policy_doc["Resource"] = resources
         clean_policy["Statement"].append(policy_doc)
         suffix = f"{a_type}{resource_name}"[:(118)]
         resource_policies[a_type] = IamPolicy(
@@ -49,22 +89,27 @@ def generate_resource_permissions(resource_name, policies, arn):
     return resource_policies
 
 
-def add_iam_policy_to_service_task_role(
-    service_template, resource, perms, access_type, services
-):
+def add_iam_policy_to_service_task_role(family, resource, perms, access_type, services):
     """
     Function to expand the ECS Task Role policy with the permissions for the resource
-    :param troposphere.Template service_template:
+    :param ecs_composex.common.compose_services.ComposeFamily family:
     :param resource:
     :param perms:
     :param access_type:
     :param list services:
     :return:
     """
-    containers = define_service_containers(service_template)
+    containers = define_service_containers(family.template)
     policy = perms[access_type]
-    task_role = service_template.resources[TASK_ROLE_T]
-    task_role.Policies.append(policy)
+    policy_title = f"{family.logical_name}{access_type}To{resource.mapping_key}{resource.logical_name}"
+    if policy_title not in family.template.resources:
+        res_policy = PolicyType(
+            policy_title,
+            PolicyName=policy.PolicyName,
+            PolicyDocument=policy.PolicyDocument,
+            Roles=[Ref(family.task_role.name["ImportParameter"])],
+        )
+        family.template.add_resource(res_policy)
     for container in containers:
         for service in services:
             if container.Name == service.name:
@@ -99,7 +144,7 @@ def map_service_perms_to_resource(
     """
     Function to
     :param resource:
-    :param family:
+    :param ecs_composex.common.compose_services.ComposeFamily family:
     :param services:
     :param str access_type:
     :param value: The value for main attribute, used for env vars
@@ -116,8 +161,17 @@ def map_service_perms_to_resource(
     resource.generate_resource_envvars()
     containers = define_service_containers(family.template)
     policy = res_perms[access_type]
-    task_role = family.template.resources[TASK_ROLE_T]
-    task_role.Policies.append(policy)
+    policy_title = (
+        f"{family.logical_name}To{resource.mapping_key}{resource.logical_name}"
+    )
+    if policy_title not in family.template.resources:
+        res_policy = PolicyType(
+            policy_title,
+            PolicyName=policy.PolicyName,
+            PolicyDocument=policy.PolicyDocument,
+            Roles=[Ref(family.task_role.name["ImportParameter"])],
+        )
+        family.template.add_resource(res_policy)
     for container in containers:
         for service in services:
             if container.Name == service.name:
