@@ -1,4 +1,4 @@
-﻿#  -*- coding: utf-8 -*-
+#  -*- coding: utf-8 -*-
 # SPDX-License-Identifier: MPL-2.0
 # Copyright 2020-2021 John Mille <john@compose-x.io>
 
@@ -6,7 +6,6 @@
 Functions to pass permissions to Services to access S3 buckets.
 """
 import re
-import warnings
 
 from compose_x_common.aws.kms import (
     KMS_ALIAS_ARN_RE,
@@ -44,7 +43,7 @@ def assign_service_permissions_to_bucket(bucket, family, services, access, value
             arn=arn,
         )
         add_iam_policy_to_service_task_role(
-            family.template, bucket, bucket_perms, access[bucket_key], services
+            family, bucket, bucket_perms, access[bucket_key], services
         )
     if keyisset(objects_key, access):
         objects_perms = generate_resource_permissions(
@@ -53,7 +52,7 @@ def assign_service_permissions_to_bucket(bucket, family, services, access, value
             arn=Sub("${BucketArn}/*", BucketArn=arn),
         )
         add_iam_policy_to_service_task_role(
-            family.template,
+            family,
             bucket,
             objects_perms,
             access[objects_key],
@@ -63,13 +62,10 @@ def assign_service_permissions_to_bucket(bucket, family, services, access, value
         ssl_perms = generate_resource_permissions(
             f"SslBucketObjefsAccess{bucket.logical_name}",
             ACCESS_TYPES[ssl_key],
-            arn=[
-                arn,
-                Sub("${BucketArn}/*", BucketArn=arn),
-            ],
+            arn=arn,
         )
         add_iam_policy_to_service_task_role(
-            family.template,
+            family,
             bucket,
             ssl_perms,
             ssl_key,
@@ -296,7 +292,7 @@ def define_lookup_buckets_access(bucket, target, services):
             arn=FindInMap("s3", bucket.logical_name, "Arn"),
         )
         add_iam_policy_to_service_task_role(
-            target[0].template,
+            target[0],
             bucket,
             bucket_perms,
             access[bucket_key],
@@ -312,7 +308,7 @@ def define_lookup_buckets_access(bucket, target, services):
             ),
         )
         add_iam_policy_to_service_task_role(
-            target[0].template,
+            target[0],
             bucket,
             objects_perms,
             access[objects_key],
@@ -322,19 +318,10 @@ def define_lookup_buckets_access(bucket, target, services):
         ssl_perms = generate_resource_permissions(
             f"SslBucketObjefsAccess{bucket.logical_name}",
             ACCESS_TYPES[ssl_key],
-            arn=[
-                Sub(
-                    "${BucketArn}",
-                    BucketArn=FindInMap("s3", bucket.logical_name, "Arn"),
-                ),
-                Sub(
-                    "${BucketArn}/*",
-                    BucketArn=FindInMap("s3", bucket.logical_name, "Arn"),
-                ),
-            ],
+            arn=FindInMap("s3", bucket.logical_name, "Arn"),
         )
         add_iam_policy_to_service_task_role(
-            target[0].template,
+            target[0],
             bucket,
             ssl_perms,
             ssl_key,
@@ -366,7 +353,7 @@ def assign_lookup_buckets(bucket, mappings):
                     arn=FindInMap("s3", bucket.logical_name, "KmsKey"),
                 )
                 add_iam_policy_to_service_task_role(
-                    target[0].template,
+                    target[0],
                     bucket,
                     kms_perms,
                     "EncryptDecrypt",
@@ -385,11 +372,19 @@ def s3_to_ecs(resources, services_stack, res_root_stack, settings):
     :param ecs_composex.common.settings.ComposeXSettings settings: ComposeX Settings for execution
     :return:
     """
-    new_resources = [
-        resources[res_name]
-        for res_name in resources
-        if resources[res_name].cfn_resource
-    ]
+    if hasattr(res_root_stack, "is_void") and not res_root_stack.is_void:
+        new_resources = [
+            resources[res_name]
+            for res_name in resources
+            if resources[res_name].cfn_resource
+        ]
+        for res in new_resources:
+            LOG.info(f"Creating {res.name} as {res.logical_name}")
+            handle_new_resources(
+                res,
+                services_stack,
+                res_root_stack,
+            )
     lookup_resources = [
         resources[res_name]
         for res_name in resources
@@ -400,13 +395,6 @@ def s3_to_ecs(resources, services_stack, res_root_stack, settings):
         for name in resources
         if resources[name].use and resources[name].mappings
     ]
-    for res in new_resources:
-        LOG.info(f"Creating {res.name} as {res.logical_name}")
-        handle_new_resources(
-            res,
-            services_stack,
-            res_root_stack,
-        )
     for res in lookup_resources:
         assign_lookup_buckets(res, settings.mappings[RES_KEY])
     for res in use_resources:
